//*****************************************************************************
//
//	SwapView.m.  
//
//	I have condensed, optimized and reformatted the code so that a swapview
//	can be implemented with this source file and a nib.  Added initPopUp 
// 	method.  Ideally, this class should hold all of the panel id's as well as //	their titles in an a List.
//
//		changes 5/25/95	by Felipe A. Rodriguez
//		changes 7/29/95	by Felipe A. Rodriguez
//			-- replaced addSubview in SwapIt method with replaceSubview.  This
//				corrected a bug where textfields remained active after swap
//
//	Most of this file was derived from:
//	
//			SwapView.m and InspectorController.m
//			by Greg Burd
//			SwapItDemo, ver 2.0
//
//	This code is supplied "as is" the author's makes no warranty as to its 
//	suitability for any purpose.  This code is free and may be distributed 
//	in accordance with the terms of the:
//		
//			GNU GENERAL PUBLIC LICENSE
//			Version 2, June 1991
//			copyright (C) 1989, 1991 Free Software Foundation, Inc.
// 			675 Mass Ave, Cambridge, MA 02139, USA
//
//*****************************************************************************

#import "SwapView.h"
#import <appkit/Window.h>
#import <appkit/Application.h>
#import <dpsclient/psops.h>
#import <dpsclient/wraps.h>
#import <dpsclient/dpsNeXT.h>
#import <appkit/graphics.h>
#import <objc/List.h>
#import <objc/objc.h>
#import <appkit/PopUpList.h>
#import <apps/InterfaceBuilder.h>


@implementation SwapView

//*****************************************************************************
//
// 		generate the pop up buttons and load their titles
//
//	1.	popUpCover must be attached to the trigger button of a popUpList in IB
//	2.	IB popUpList should have only one menu cell
//	3.	this method should be called from an awakefromnib with a NULL 		
//		terminated array of char str's   e.g. char *list[3] = {"1", "2", NULL}		
//
//*****************************************************************************

- initPopUp:(char **)titlesArray
{
	if(popUpCover)			// if popUpCover is hooked up to a popuplist in IB
		{							// reset popup to point to PopUpList, in IB
			 						// it is set to point to the trigger button
		if (![popup isKindOf:[PopUpList class]]) 
			popup = [popUpCover target];			
		[popup removeItemAt:0];				// remove the top placeholder cell
		[popUpCover setTitle:*titlesArray];		// set name of trigger button
		theTitle = *titlesArray;	
		while(*titlesArray)
			[[[popup addItem:*titlesArray++] 
							setTarget:self]setAction:@selector(popUpAction:)];
		}	
	[self swapIt];

	return self;
}
//*****************************************************************************
//
// 		designated initilizer for swapviews
//
//*****************************************************************************

- initFrame:(NXRect *)theFrame
{
char title[] = {"FirstInspector"};

	[super initFrame:theFrame];
	
	lastInspector = NULL;
	currentInspector = NULL;
	backgroundGray = NX_LTGRAY;
	
		/* swap in the first inspector */
	if(popUpCover)
		theTitle = [popUpCover title];
	else
		theTitle = title;	
	[self setBackgroundGray:NX_LTGRAY];		// default
	[self swapIt];						// now swap in the first inspector
		/* set up the matrix so that it can receive the command-key events */
	//[[[popUpCover window] contentView] setNextResponder:[popUpCover target]];

	return self;
}
//*****************************************************************************
//
// 		swaps views in and out of inspector's swapview
//
//*****************************************************************************
 
- swapIt:sender
{ 
	[self swapIt]; return self; 
}
- swapIt
{
id inspector; 

	inspector = [self whatPanel];
	if(lastInspector != inspector)	// make sure this is a new view to swap in  
		{
		if(!inspector)				// no inspector so clean up the background 
			{
			[self lockFocus];
			PSsetgray(backgroundGray);			// use the backgroundGray
			NXRectFill(&bounds);
			[self unlockFocus];
			lastInspector = NULL;			// now the last inspector is NULL 
			NXPing();			// let the window server cetch up... (yawn...)
			}
				//  add the subview to SwapView and its responder chain		
		if(lastInspector)
			[self replaceSubview:[lastInspector contentView] 
												with:[inspector contentView]];
		else
			[self addSubview:[inspector contentView]];
		[self display];
		lastInspector = inspector;		// remember the panel it came from 	
		}

	return self;
}

- free
{
	return [super free];
}
//*****************************************************************************
//
// 		This should return the id of a panel, which is off screen, and 
//		buffered, but not defered (SwapView needs its gstate).  SwapView will 
//		take the contentView of the panel and swap it in while also removing 
//		the old view, if any, and placing it back into its old panel.
//
//	 WARNING!!!!
//	Don't forget to make sure that the off screen windows are NOT deferred!!!
//	This means make sure that you turn the Deffered switch OFF in IB when
//	looking at the Atributes inspector.  This allows offscreen drawing.  
//	If it was on, then the gstate will be zero!!!!
//
// 		HINT::  these panels should all be nibs so they have controllers
//				and the above code should load the nib section if it
//				already isn't loaded.  It should load here because you only
//				want to take up memory when it is demanded. 
//
//*****************************************************************************

- whatPanel
{
	if(theTitle) 		// find out what panel to give to the SwapView
		{
				// make sure that the Button title matches the inspector  
				// name this doesn't happen with a command key call
//		if (strcmp(theTitle,[popUpCover title])) 
//			[popUpCover setTitle:theTitle];
		
		if (!strcmp(theTitle,"FirstInspector"))		// for use by swapViews
			return firstInspectorPanel;				// w/o a popUpList
		
		if (!strcmp(theTitle,"SecondInspector"))	// for use by swapViews
			return secondInspectorPanel;			// w/o a popUpList

		if (!strcmp(theTitle,"defaults"))		// hardwired name of inspector
			return firstInspectorPanel;
		
		if (!strcmp(theTitle,"Editor"))			// hardwired name of inspector
			return secondInspectorPanel;
		}
								// the button title has no related inspector, 
	return (id)NULL;			// this should never happen.	
}

//*****************************************************************************
//
// 		this is to allow us to junp to any inspector given its key name 
//
//*****************************************************************************

- inspectName:(char *)str
{
	if (str) 
		theTitle = str; 
	[self swapIt];
	
	return self;
}
//*****************************************************************************
//
// 		target of pop up list button 
//
//*****************************************************************************

- popUpAction:sender
{
			// set up a pointer to the selectedCell of the matrix 
	 theTitle = [[sender selectedCell] title];
	 [self swapIt];	 						// swap out the view

	return self;
}
- currentInspector
{
	return currentInspector;
}

- lastInspector
{
	return lastInspector;
}

- (float)backgroundGray
{ 
	return backgroundGray; 
}

- setBackgroundGray:(float)aGray
{
	backgroundGray = aGray;
	return self;
}

@end

