//************************************************************************
//
//	ToolBar.m.  
//
//	by	Felipe A. Rodriguez		
//
//
//	This code is supplied "as is" the author makes no warranty as to its 
//	suitability for any purpose.  This code is free and may be distributed 
//	in accordance with the terms of the:
//		
//			GNU GENERAL PUBLIC LICENSE
//			Version 2, June 1991
//			copyright (C) 1989, 1991 Free Software Foundation, Inc.
// 			675 Mass Ave, Cambridge, MA 02139, USA
//
//************************************************************************


#import "ToolBar.h"
#import "ToolBarView.h"
#import "ToolIconView.h"
#import "HLRecord.h"
#import "Coordinator.h"
#import <appkit/appkit.h>
#import <dbkit/DBImageView.h>

#import <sys/types.h>
#import <sys/dir.h>  /* POSIX applications #include <dirent.h> */

@implementation ToolBar

//************************************************************************
//
//		sent to us after nib objects are unarchived and init'd 
//
//************************************************************************

- awakeFromNib
{
NXRect	appIconFrame;
NXRect	lineViewRect;
id appIcon;
char *sC;
id image;

	appTile = [NXImage findImageNamed: "NXAppTile"];
	appIcon = [NXApp appIcon];					// get appIcon window
    [appIcon getFrame:&appIconFrame];			// get appIcon frame rect
    [Window getContentRect:&lineViewRect 		// get content view rect appIco
						forFrameRect:&appIconFrame style:[appIcon style]];
				// reduce the size and width/ht by a border of 1
	NX_X(&lineViewRect) = 1;
	NX_Y(&lineViewRect) = 1;
	NX_WIDTH(&lineViewRect) -= 1 ;
	NX_HEIGHT(&lineViewRect) -= 1;
															// create tool view
	theToolView = [[ToolBarView alloc] initFrame:&lineViewRect];	
															// create icon view
	toolIconView = [[ToolIconView alloc] initFrame:&lineViewRect];	
    toolWin = [[Window allocFromZone:[self zone]]			// create tool Win
				   	initContent:&appIconFrame
				   	style:NX_TOKENSTYLE
				   	backing:NX_BUFFERED 
					buttonMask:NX_CLOSEBUTTONMASK defer:NO];
	[toolWin setFreeWhenClosed:NO];

	if(![toolWin setFrameUsingName:"toolBar"])			
		[toolWin moveTo:100.0 :100.0];					// initial pos of toolB
	[toolWin setFrameAutosaveName:"toolBar"];			// remember location
    [[toolWin setContentView:theToolView] free];		// replace cont w/toolV
									// add IB created backWindow's contentView
	[[toolWin contentView] addSubview:[backWindow contentView]];
	toolView = [toolWin setContentView:toolIconView];		
	[toolWin setBackgroundGray:NX_BLACK];
	[[toolWin contentView] getFrame:&lineViewRect];						
	[[toolWin contentView] convertRect:&lineViewRect toView:nil];
	[toolWin setTrackingRect:&lineViewRect					// create mouse 
					inside:YES								// tracking rect
					owner:self
					tag:1500
					left:NO
					right:NO];

				//	cell selection in hotList should persist between sessions
				//	so recall previous selection
	sC = NXCopyStringBuffer(NXGetDefaultValue([NXApp appName],"selectedCell"));		
	selCell = atoi(sC);
	free(sC);
	[HLBrowser loadColumnZero];		// NXBrowser s/load cols from its delegate
	mat = [HLBrowser matrixInColumn:0];		
	if(selCell < [dirList count])
		{
		[mat selectCellAt:selCell :0];
		if(image = [[dirList objectAt:selCell] gateIcon])
			{
			[HLImageView setImage:image];
			[toolIconView setImage:image];
			}
		[mat scrollCellToVisible:selCell :0];
		}

	[toolWin display];
    [toolWin orderFront:self];			

    return self;
}
//*****************************************************************************
//
// 		user pressed play button in toolbar 
//
//*****************************************************************************

- playLink:sender 
{
	[sender setState:0];									
	[[NXApp delegate] Link:self];							

	return self;
}
- playLink 
{
	if([dirList count] >= 1)					// open sel'td HotList file
		[[NXApp delegate] app:self openFile:[[dirList objectAt:selCell] 
							gateDocPath] type:NULL];
	else						// if HotList path not set use LinkOptions file
		[[NXApp delegate] app:self openFile:[[NXBundle mainBundle] directory]
							type:NULL];	
	wasHidden =	NO;							// if bringing up link do not hide

	return self;
}
//*****************************************************************************
//
// 		user pressed stop button in toolbar 
//
//*****************************************************************************

- playUnlink:sender 
{
	[sender setState:0];
	if([[[NXApp mainMenu] findCellWithTag:3] isEnabled])	// is unlink menu
		[[NXApp delegate] UnLink:self];						// item enabled
	
	return self;
}
//*****************************************************************************
//
// 		go to next gate doc in hotList 
//
//*****************************************************************************

- nextItem:sender 
{
id image;

	[sender setState:0];
	if(selCell < ([dirList count] - 1))
		selCell++;
	else
		selCell = 0;
	[mat selectCellAt:selCell :0];
	if(image = [[dirList objectAt:selCell] gateIcon])
		{
		[HLImageView setImage:image];
		[toolIconView setImage:image];
		}
	[mat scrollCellToVisible:selCell :0];

	return self;
}
//*****************************************************************************
//
// 		go to previous gate doc in hotList 
//
//*****************************************************************************

- prevItem:sender 
{
id image;

	[sender setState:0];
	if(selCell > 0)
		selCell--;
	else
		selCell = [dirList count] - 1;
	[mat selectCellAt:selCell :0];
	if(image = [[dirList objectAt:selCell] gateIcon])
		{
		[HLImageView setImage:image];
		[toolIconView setImage:image];
		}
	[mat scrollCellToVisible:selCell :0];

	return self;
}
//*****************************************************************************
//
// 		provide to listBrowser(NXBrowser) it's data to be displayed
//			do this because we are its delegate  
//
//*****************************************************************************

- (int)browser:b fillMatrix:m inColumn:(int)c
{
int	i = 0, strSize;
id t, obj = nil;
char *ptr;
struct direct *dirp;
DIR *dp;
NXHashState state;
const void  *key; 
void  *value; 

    if ((dp = opendir(NXGetDefaultValue([NXApp appName], "HotList"))) != NULL)
		{
		if(!namHashTable)				// alloc hashtable w/str keys, obj vals
			namHashTable = [[HashTable alloc] initKeyDesc:"*" valueDesc:"@"];
		else
			tmpHashTable = [namHashTable copyFromZone:[self zone]];
		[namHashTable empty];
		if(dirList)
			[dirList empty];					// empty list
		else
			dirList = [[List alloc] init];		// produce an array of objects
		while (dirp = readdir(dp)) 				// read dir and find all
			{									// entries with ext .Gate
			if((strSize = strlen(dirp->d_name)) > 5)
				{
				ptr = dirp->d_name + strSize;		// point to end of name str
				ptr -= 5;						
				if(strcmp(ptr, ".Gate") == 0)		// if extension is .Gate
					{								// if str not in hash table
					if ([tmpHashTable isKey:dirp->d_name] == NO) 	
						{							// create a HotList record
						obj = [[HLRecord alloc] init];	
						[obj setGateDocName:dirp->d_name];
						[namHashTable insertKey:dirp->d_name value:obj];
						}
					else							// if record exists in hash
						[namHashTable insertKey:dirp->d_name 	
								value:[tmpHashTable valueForKey:dirp->d_name]];
					}
				}
			};
		[tmpHashTable free];
									// sort and display dir in NXBrowser
		for(i = 0; i < [namHashTable count]; i++) 
			{						// Return NXHashState structure required 
									// when iterating through the HashTable
			state = [namHashTable initState]; 
			while ([namHashTable nextState:&state key:&key value:&value]) 
				{					// iterate thru the hashtable and perform
				if ([dirList indexOf:value] == NX_NOT_IN_LIST)
					{
					if(!obj)
						obj = value;					// an ascending sort
					if(strcasecmp(key, [obj gateDocName]) < 0)		
						obj = [namHashTable valueForKey:key];
					}			
				};
			[m addRow];								// add row to HotList matrx
			t = [m cellAt:i :0];					// ret cell in NXBrowser
			[dirList addObject:obj];
			[t setStringValueNoCopy:[obj gateDocName]];
			[t setLoaded:YES];
			[t setLeaf:YES];
			obj = nil;
			}
		hotListDirValid = YES;						// hotList dir was opened
		}
	else
		{
		NXRunAlertPanel(0, "Error opening HotList directory", 0, 0, 0);
		hotListDirValid = NO;						// hotList dir is invalid
		}

    return i; 
}
//*****************************************************************************
//
// 		close toolbar 
//
//*****************************************************************************

- close:sender 
{
	[sender setState:0];
	[toolWin close];

	return self;
}
//*****************************************************************************
//
// 		mouse has entered our tracking rect (window)
//
//*****************************************************************************

- mouseEntered:(NXEvent *)theEvent 
{
id image;

	wasHidden =	[NXApp isHidden];						// is the app hidden?
	[toolWin setContentView:toolView];		
	[toolWin display];	
	[HLBrowser loadColumnZero];		// NXBrowser s/load cols from its delegate
	if(hotListDirValid)				// if hotList path is valid
		{
		mat = [HLBrowser matrixInColumn:0];		
		[HLBrowser useScrollButtons:NO];		
		[mat selectCellAt:selCell :0];
		if(image = [[dirList objectAt:selCell] gateIcon])  	// if Gate doc
			[HLImageView setImage:image];				 // contains an image
		[mat scrollCellToVisible:selCell :0];
		[HLBrowser display];
		[hotList center];
		[hotList makeKeyAndOrderFront:self];
		}

	return self;
}
//*****************************************************************************
//
// 		mouse has exited our tracking rect (window)
//
//*****************************************************************************

- mouseExited:(NXEvent *)theEvent 
{
char tmpStr[16];

	[toolWin setContentView:toolIconView];		
	[toolWin display];		
	[hotList close];

	sprintf(tmpStr, "%d", selCell);
								// write an entry to defaults database
	if(!NXWriteDefault([NXApp appName], "selectedCell", tmpStr))
		NXRunAlertPanel(0, "Error setting defaults database", 0, 0, 0);
	if(wasHidden)				// if app was hidden prior to cursor in, hide
		[NXApp hide:self];

	return self;
}
- (BOOL)acceptsFirstMouse
{
	return YES;
}
//*****************************************************************************
//
// 		forward anything in Greek to someone who might understand 
//
//*****************************************************************************

- forward:(SEL)aSelector :(marg_list)argFrame
{
    if ( [toolWin respondsTo:aSelector] )
        return [toolWin performv:aSelector :argFrame];

    [self doesNotRecognize:aSelector];					// error abort process

	return self;
}
//************************************************************************
//
// 	free simply gets rid of everything we created
// 	This is how nice objects clean up.
//
//************************************************************************

- free
{
	if(dirList)
		{
		[dirList freeObjects];							// free list's objects
		[dirList free];									// free list object
		}		
	if(toolWin)
		[toolWin free];									
	if(theToolView)
		[theToolView free];									
	if(toolIconView)
		[toolIconView free];									
	if(namHashTable)
		[namHashTable free];									

    return [super free];
}

@end
