//************************************************************************
//
//	Portions of this file were derived from:
//	 
//			Coordinator.m      
//			by Joe Freeman, David LaVallee   
//			Subprocess Example, Release 2.0 
//			NeXT Computer, Inc.
//
//	Numerous additions, deletions, changes and any bugs courtesy of:
//
//		Felipe A. Rodriguez		
//
//	This code is supplied "as is" the author makes no warranty as to its 
//	suitability for any purpose.  This code is free and may be distributed 
//	in accordance with the terms of the:
//		
//			GNU GENERAL PUBLIC LICENSE
//			Version 2, June 1991
//			copyright (C) 1989, 1991 Free Software Foundation, Inc.
// 			675 Mass Ave, Cambridge, MA 02139, USA
//
//************************************************************************

#import "Parse.h"
#import "OptionsEditor.h"
#import "options.h"
#import "IconView.h"
#import "ToolBar.h"
#import "DOserver.h"
#import "Timer.h"
#import "InactivityTimer.h"
#import "Coordinator.h"
#import "CommandScroll.h"
#import "Subprocess.h"
#import <appkit/nextstd.h>
#import <appkit/Application.h>
#import <dpsclient/dpsNeXT.h>
#import <objc/NXStringTable.h>
#import <defaults/defaults.h>
#import <bsd/c.h>
#import <sys/dir.h>  /* POSIX applications #include <dirent.h> */

#define	BUFFSIZE 56
#define	firstTime "firstTime.2.0.6" 		


		// used in producing timed events
static void Animate();
	static DPSTimedEntry aniTimeTag;

@interface Coordinator(Private)

		// preferences controls 
	extern char *cList[];
	extern char *rfList[];
	extern char *rsList[];

		// list for Gate doc save path's
	static char *olist[] = {"/options", "/pppup", "/Icon.tiff", NULL};
	const char *filter[2] = {"Gate", NULL};

		// private functions 
- miscPrep;
- miscPrep2;

		// dial on demand server 
#import <mach/mach_error.h>
#import <servers/netname.h>
#import "nsRPC_types.h"


any_t server_loop(any_t port);

					// defined by MiG: 
boolean_t nsRPC_server(msg_header_t *in, msg_header_t *out);

				// from Request types in nsRPCUser.c 
struct message 
		{                
		msg_header_t    head;        /* standard header field */
		msg_type_t      arg1_type;   /* first arg type */
		int             arg1;        /* first arg */
		msg_type_t      arg2_type;   /* second arg type */
		int             arg2;        /* second arg */
		};

@end
@implementation Coordinator(Private)

//*****************************************************************************
//
// 		misc to do before coordinator is ready to start a subprocess 
//
//*****************************************************************************

- miscPrep
{
	if(!aniTimeTag)  
		aniTimeTag = DPSAddTimedEntry(			// register function Animate
			0.2, 								// to be called every period of 		
			(DPSTimedEntryProc)Animate, 		// arg0
			(id)self, 
			NX_BASETHRESHOLD);

	return self;
}
//*****************************************************************************
//
// 		misc to do before coordinator starts a subprocess 
//
//*****************************************************************************

- miscPrep2
{
int i = 0;

	standbyGate = NO;				// tells animate to stop stndby icon seq		
	[link setEnabled:NO];						// disable link button
	[dial setEnabled:NO];						// disable dial button
	[theOpenButton setEnabled:NO];			// disable the open doc button
	acceptAnother = NO;					// we will not open another .Gate doc
	while(i < 3)					
		{										// erase captured str's array
		if(capturedStr[i])						
			free(capturedStr[i]);
		capturedStr[i++] = NULL;				// blank, for next invocation
		}
	linkStg = 0;	// prevent stages in link process from getting called twice
									
					// setup and load our keyword's hashtable
	if(strHashTable)
		free(strHashTable);
	strHashTable = [[HashTable alloc] initKeyDesc:"*" valueDesc:"i"];
	for(i = 0; i < 2; i++)
		{
		numEntries[i] = i;
		[strHashTable insertKey:NXGetDefaultValue([NXApp appName], cList[i+1]) 
													value:&numEntries[i]];
		}
	for(i = 0; i < 4; i++)
		{
		numEntries[i + 2] = i + 2;
		[strHashTable insertKey:NXGetDefaultValue([NXApp appName], rfList[i]) 
													value:&numEntries[i + 2]];
		}
	numEntries[6] = 6;
	[strHashTable insertKey:"address" value:&numEntries[6]];
	numEntries[7] = 7;
	[strHashTable insertKey:"remote" value:&numEntries[7]];

	return self;
}

@end
@implementation Coordinator

//************************ Subprocess Delegation *************************
//************************************************************************
//
// 		recieves output from subprocess
//		canonicalizes output stream into lines
//
//************************************************************************
 
- subprocessOutput:(char *)buffer
{
static char strbuf[132];
static int strpos = 0;

	while(*buffer)		
		{			// cr/lf delimit lines and we do not exceed strbuf size		
		if((*buffer != '\n' && *buffer != '\r') && strpos < 128)	
			{
			strbuf[strpos++] = *buffer++; 
			strbuf[strpos] = '\0';
			}
		else 
			{
			while((*buffer == '\n' || *buffer == '\r') && (strpos < 130))
				strbuf[strpos++] = *buffer++;
			strbuf[strpos] = '\0';
			[self analyzeBuffer:strbuf];
			strpos = 0; 
			}
		}
		
    return self;
}
//************************************************************************
//
//		- handles secure mode if switch is set
//		- captures certain keywords as status and info markers
//		- calls gotIt, displayLinkStatus methods when link is established
//		- sends data stream to diagnostics window
//
//************************************************************************
 
- analyzeBuffer:(char *)buffer
{
static char *str, delim[] = {" \n\r"}, *argString = NULL, *String;
int *cntr;

	if(argString != NULL)
		free(argString);
	argString = NXCopyStringBuffer(buffer);
	if(strcmp(NXGetDefaultValue([NXApp appName], "DisplayDiagnostics"),"YES") 	
		== 0)
		{
		String = buffer;
		if(strcmp(NXGetDefaultValue([NXApp appName], "secure"),"YES") == 0)
			while(*buffer){*buffer++ = 'X';} 			// secure display 
		[commandView appendString:String];
		}
	str = strtok(argString, delim);					// Parse string and test 
	while(str != NULL)								// it before sending it to
		{											// main window
		if ([strHashTable isKey:str] == YES) 		// if str is in hash table
			{
			cntr = [strHashTable valueForKey:str];
			switch(*cntr)
				{							
				case 2: 						// redial?
				case 3: 						// Persistent connect
				case 4:							// Fail connect
				case 5:							// LCP terminate
					[self UnLink:nil];		// if respective switch is enabled 
					if((strcmp(NXGetDefaultValue([NXApp appName], 
											rsList[*cntr - 2]),"YES") == 0))
						{
						if(lastCall[0] != '\0')					// redial
							[self app:self openFile:lastCall type:NULL];
						}
					break;
				
				case 0:  								// baud str caught
					if((str = strtok(NULL, delim)) && (capturedStr[0] == NULL))
						capturedStr[0] = NXCopyStringBuffer(str);
					break;
	
				case 1:  								// the link is up
					if(linkStg < 4)						// if ivar is >, ppp
						{
						[self gotIt];					// may be sending
						linkStg = 4;					// keyword twice
						}
					break;								

				case 6:								// local address str caught
					if((str = strtok(NULL, delim)) && (linkStg < 5))	
						{		
						capturedStr[1] = NXCopyStringBuffer(str);
						linkStg = 5;	
						}
					break;
										
				case 7:  							// display addresses
					if((strcmp(NXGetDefaultValue([NXApp appName], 	
						"DisplayDiagnostics"),"YES") == 0) && (linkStg < 6))
						{
						if((str = strtok(NULL, delim)) != NULL)
							{
							if((str = strtok(NULL, delim)) != NULL)
								{
								if((str = strtok(NULL, delim)) != NULL)
									{
									capturedStr[2] = NXCopyStringBuffer(str);
									[self displayLinkStatus];
									}
								}
							}
						}
					else
						[iTimer pppstats:nil];		// run pppstats w/o aView
					break;

    			default:	 		
					break;
				}
			}
		str = strtok(NULL, delim);
		}
		
    return self;
}
//************************************************************************
//
// 		Called when subprocess has terminated
//
//************************************************************************
 
- subprocessDone
{
	[theTimer stopTimer];
	if(theGateServer)
		[theGateServer free];			
	theGateServer = nil;
	[theSubprocess free];
	theSubprocess = nil;
	if(userWantsTermination)	// end only if user chose too (not subP fail)
		[NXApp terminate:self];
	else											// if not exiting	
		[self miscPrep];							// misc initialization's
	acceptAnother = YES;					
	
    return self;
}

- subprocessError:(const char *)errorString
    // uses stringTable to localize error messages as appropriate,
    // and then displays the message in an alert panel
{
const char *returnedString;
    
    if (returnedString = [stringTable valueForStringKey:errorString])
		NXRunAlertPanel(0, returnedString, 0, 0, 0);
    else
		NXRunAlertPanel(0, errorString, 0, 0, 0);

    return self;
}
//************************************************************************
//
// 		Called when link is established
//
//************************************************************************
 
- gotIt
{
	[self removeTimedEntry];
							// ppp is up, play snd for me
	if(strcmp(NXGetDefaultValue([NXApp appName],"sound"), "YES") == 0) 
		[[[Sound findSoundFor: "majestic"] play: nil] free];
	[theTimer Fire:self];
	if(!onImage)
		onImage = [NXImage findImageNamed:"g4"];
	[theIconView setImage:onImage];
					// if auto hide switch is set wait 5 sec before hiding	
	if(strcmp(NXGetDefaultValue([NXApp appName], "autoHide"),"YES") == 0)	
		[NXApp perform:@selector(hide:) with:self afterDelay:5000 
															cancelPrevious:NO];				

    return self;
}
//************************************************************************
//
// 		displays captured info regarding current ppp session
//
//************************************************************************
 
- displayLinkStatus
{
	if(capturedStr[0] != NULL)					// in case we man dialed
		{	// we will not know connect speed
		[commandView appendString:"\n#########################\n"];
		[commandView appendString:"####  Connected at:  "];
		[commandView appendString:capturedStr[0]];
		}
	[commandView appendString:"\n#########################\n"];
	[commandView appendString:"####  local IP address is:  "];
	[commandView appendString:capturedStr[1]];
	[commandView appendString:"\n#########################\n"];
	[commandView appendString:"####  remote IP address is:  "];
	[commandView appendString:capturedStr[2]];
	[commandView appendString:"\n#########################\n"];
	linkStg = 6;									// ppp link is up, set ivar
	[iTimer pppstats:commandView];					// run pppstats w/view

    return self;
}
//*********************** Application Object Delegation ***********************
//*****************************************************************************
//
// 			setup defaults data base cache before app is init'd
//
//*****************************************************************************

- appWillInit:sender
{
    static NXDefaultsVector myDefaults = {			// setup defaults database
        {"DisplayDiagnostics", "NO"},				
		{"secure", "NO"},			
		{"autoLaunch", "NO"},			
		{"autoHide", "YES"},				// auto hide upon link		
		{"autoRedial", "YES"},			
		{"PersistCon", "YES"},			
		{"FailRedial", "YES"},				// redial if a Fail occurs
		{"LCPterminate", "YES"},			// redial if lcp termination occurs
		{"Fail", "Exit."},			
		{"reDial", "Failed"},			
		{"Persist", "down"},					// persistent connect trigger
		{"LCPterm", "terminated."},				// lcp termination trigger
		{"DispPPP", "YES"},						// display pppstats
		{"toolbar", "YES"},			
		{"DoD", "NO"},							// dial on demand
		{"useFIFO", "YES"},						// use a FIFO for IPC w/pppd?
		{"sound", "YES"},						// play sound
		{"iTimeout", "Off"},					// inactivity unlink's us
		{"preTimeout", "YES"},					// inactivity unlink's us
		{"iTimeThreshold", "0"},				// inactivity threshold 
		{"selectedCell", "0"},					// cell selected in hotList
		{firstTime, "YES"},						// first time thru?
		{"options", "/etc/ppp/options"},			
		{"ip-down", "/etc/ppp/ip-down"},			
		{"ip-up", "/etc/ppp/ip-up"},			
		{"resolv", "/etc/resolv.conf"},			
		{"syslog", "/etc/syslog.conf"},			
		{"messages", "/usr/adm/messages"},			
		{"rc", "/etc/rc.local"},			
		{"remote", "/etc/remote"},			
		{"locFIFO", "/usr/adm/ppp2.2.debug"},				// pref form start
		{"comLine", "/usr/local/bin/pppd -detach"},			
		{"chatLine", "/usr/local/bin/chat -v -f"},			
		{"HotList", "/"},			
		{"BaudStr", "CARRIER"},								// pref form end
		{"LinkUp", "local"},						// Link is up trigger
		{"tipHost", "b288"},			// name of tip Host from /etc/remote
		{"dialPrefix", "ATD"},			// modem AT command prefix for dialing
		{"modemPort", "cufa"},			// modem port, used in releasing locks
		{"path", "/"},							// path for .Gate documents
		{"lastNumDialed", "8675309"},				// default man dial number
		{"savedTime", "000000000000"},			// save time used this mo
		{"monthTime", "000000000000"},			// save mo,yr of this ses'n
        {NULL}};									// make local cache of ddb
    NXRegisterDefaults([NXApp appName], myDefaults);	
	acceptAnother = YES;
 
    return self;
}
//*****************************************************************************
//
// 			standard init
//
//*****************************************************************************

- appDidInit:sender
{
NXRect	appIconFrame;
NXRect	lineViewRect;
id appIcon;
time_t waitTime;			// time to delay

	time(&ltime);								// Get time and place in time_t
		if(strcmp(NXGetDefaultValue([NXApp appName],"useFIFO"), "YES") == 0) 
	[self syslogdReset];						// reset the syslogd daemon

							// setup appIcon for animation
	appIcon = [NXApp appIcon];					// get appIcon window
    [appIcon getFrame:&appIconFrame];			// get frame rect
    [Window getContentRect:&lineViewRect 		// get content view rect appIco
						forFrameRect:&appIconFrame style:[appIcon style]];
	NX_X(&lineViewRect) = 1;					// reduce the size and width/ht 
	NX_Y(&lineViewRect) = 1;					// by a border of 1
	NX_WIDTH(&lineViewRect) -= 1 ;
	NX_HEIGHT(&lineViewRect) -= 1;
	theIconView = [[IconView alloc] initFrame:&lineViewRect];
	[[appIcon contentView] addSubview:theIconView];
	timeCell = [theIconView getTextCell]; 	// return the iconView's textCell
			
	imageList = [[List alloc] init];	// produce an array of objects
	[imageList insertObject: [NXImage findImageNamed: "g20"] at: 0];
	[imageList insertObject: [NXImage findImageNamed: "g21"] at: 1];
	[imageList insertObject: [NXImage findImageNamed: "g22"] at: 2];
	[imageList insertObject: [NXImage findImageNamed: "g23"] at: 3];
	[imageList insertObject: [NXImage findImageNamed: "g24"] at: 4];
	[imageList insertObject: [NXImage findImageNamed: "g10"] at: 5];
	[imageList insertObject: [NXImage findImageNamed: "g11"] at: 6];
	[imageList insertObject: [NXImage findImageNamed: "g12"] at: 7];
	[imageList insertObject: [NXImage findImageNamed: "g13"] at: 8];
	[imageList insertObject: [NXImage findImageNamed: "GateKeeper"] at: 9];

    if(strcmp(NXGetDefaultValue([NXApp appName], firstTime),"YES") == 0) 
		{							// if first time this version was launched
		[self showInfo:self];	
		if(!NXWriteDefault([NXApp appName], firstTime, "NO"))
			NXRunAlertPanel(0, "Error writing to defaults database", 0, 0, 0);	
		}
	if(strcmp("YES", NXGetDefaultValue([NXApp appName], "toolbar")) == 0)	
		[self toolBar:self];						// show toolbar if set

						// setup Inactivity timeout and pppstats controller
	iTimer = [[InactivityTimer allocFromZone:[self zone]] init];

	[self port];		// find out what port the modem is on
    if(strcmp(NXGetDefaultValue([NXApp appName],"NXAutoLaunch"),"YES") == 0) 
		{						// if app was dock autoLaunched
		waitTime = ltime + 2;	// delay for X sec so that syslogd can sync
		while(ltime < waitTime)	// ltime ivar is set in syslogReset method 
			{time(&ltime);}
		}
	theTimer = [[Timer allocFromZone:[self zone]] init];
   			// If we weren't asked to open any documents at launch time, then 	
			// we were launched by double clicking on the application instead 
			// of a document. If so check whether we should auto-launch w/def's
    if(!NXGetDefaultValue([NXApp appName], "NXOpen") &&
		!NXGetDefaultValue([NXApp appName], "NXOpenTemp") &&
			!NXGetDefaultValue([NXApp appName], "NXServiceLaunch"))
		{
		if(strcmp(NXGetDefaultValue([NXApp appName], "autoLaunch"),"YES") == 0)	
			[self Link:self];	
		}
	[self DialOnDemand];		// prepare for dial on demand if set
					 
    return self;
}
//*****************************************************************************
//
// 		This method is performed whenever a user double-clicks on an icon in
// 		the Workspace Manager representing a Gate program document.  
// 		
// 		Instantantiates the subprocess object which exec's pppd
//
//*****************************************************************************

- (int)app:sender openFile:(const char *)path type:(const char *)type 
{
char commandLine[MAXPATHLEN + 1]; 

	[self openMainWin];							// misc initialization's
	[self miscPrep];							// misc initialization's
	[self miscPrep2];							// misc initialization's
	if([iTimer posNonInter])
		[commandView appendString:"Dial on demand triggered ppp session\n"];
	strncpy(commandLine, NXGetDefaultValue([NXApp appName], "comLine"), 
																MAXPATHLEN);
	commandLine[MAXPATHLEN-1] = 0;
	if(path == NULL)
		{				
		lastCall[0] = '\0';
		if(![[NXBundle mainBundle] getPath:Path forResource:"options" 
																ofType:NULL])
			NXRunAlertPanel(0, "Error getting mainbundle path:app", 0, 0, 0);
		if((strlen(commandLine) + strlen(Path) + 6) < MAXPATHLEN)
			{
			strcat(commandLine, " file ");
			strcat(commandLine, Path);
			}
		}
	else
		{								
		strcpy(lastCall, path);			// stores path of last invocation
		if((strlen(NXGetDefaultValue([NXApp appName], "chatLine")) + 45 + 
					(2 * strlen(path)) + strlen(commandLine)) < MAXPATHLEN)	
			{									// if we won't overrun buffer
			strcat(commandLine, " file ");
			strcat(commandLine, path);
			strcat(commandLine, olist[0]);	// options file name wrapper
			strcat(commandLine, " connect \"");				
			strcat(commandLine, NXGetDefaultValue([NXApp appName],"chatLine"));				
			strcat(commandLine, " ");
			strcat(commandLine, path);
			strcat(commandLine, olist[1]);	// pppup file name wrapper
			strcat(commandLine, "\"");
			}
		else
			perror("command line is longer than MAXPATHLEN");
		}
	[unlink setEnabled:YES];					// enable unlink button
	theSubprocess = [[Subprocess alloc] init:commandLine withDelegate:self 	
									andStdErr:YES];
				
	return 1;
}
//*****************************************************************************
//
// 		invoked immediately after app is hidden and unhidden respectively
//
//			the following two methods are implemented to produce
//			normal main window/menu app behavior when toolBar is open
//
//******************************************************************************
- appDidHide:sender
{
	[[NXApp mainMenu] close];		// close the main menu  
									
    return self;
}

- appDidUnhide:sender
{
	[[NXApp mainMenu] makeKeyAndOrderFront:self];		 

    return self;
}
//*****************************************************************************
//
// 		Use terminal to do a manual link 
//
//*****************************************************************************

- Dial:sender  
{
	if([link isEnabled] && [self runScript:"preLink"])
		{ 
		[self miscPrep];						// misc initialization's
		[self miscPrep2];						// misc initialization's
		[unlink setEnabled:NO];					// enable unlink button
		[iTimer setPosNonInter:NO];				// user interactive session
		if(!theGateServer)					// create distributed Obj server
			theGateServer = [[DOserver alloc] init];  		
		[theGateServer setDelegate:self];			
		GateConnection = [NXConnection registerRoot: theGateServer
						withName:"GateKeeperServer"];	// DO name of server
		[GateConnection runFromAppKit];	// listen for DO messages from appKit
		if(![[NXBundle mainBundle] getPath:Path forResource:"MODEM" 
						ofType:NULL])
			NXRunAlertPanel(0, "Error getting path for MODEM file", 0, 0, 0);
		[[Application workspace] openFile:Path withApplication:"Terminal"];	
		}

    return self;
}
//*****************************************************************************
//
// 		Called: when menu item link is pressed, when DNS triggers us.  
//		Calls app:sender
//
//*****************************************************************************

- Link:sender  
{
	if([link isEnabled] && [self runScript:"preLink"])
		{ 
		if(!sender)									// DNS triggered session,
			[iTimer setPosNonInter:YES];			// possibly non-interactive 
		else										
			[iTimer setPosNonInter:NO];				// user interactive session
		if(toolBar)					// if toolBar is open use the Gate doc
			[toolBar playLink];		// selected in the HotList to launch pppd
		else						// else use LinkOptions file in App wrapper
			{										// find app's home dir
			if(!strcpy(Path, [[NXBundle mainBundle] directory]))
				NXRunAlertPanel(0,"Error getting mainbundle path:Link",0,0,0);
			else					// call app:sender to use deflt options
				[self app:self openFile:Path type:NULL];	
			}
		}			

    return self;
}
//*****************************************************************************
//
// 		Called when menu item Unlink is pressed.  Calls app:sender
//
//*****************************************************************************

- UnLink:sender  
{
	if([self runScript:"preUnLink"])
		{ 
		if(theSubprocess && [theSubprocess respondsTo:@selector(terminate:)])
			[theSubprocess terminate:sender];
		openGate = YES;			// animate should do open gate icon sequence
		standbyGate = YES;			// animate should do open standby icon seq	
		[iTimer pppstatsReset];					// disable pppstats
		[timeCell setStringValue:"      "];
		[link setEnabled:YES];					// enable link button
		[unlink setEnabled:NO];					// disable unlink button
		[theOpenButton setEnabled:YES];			// enable the open doc button
		[dial setEnabled:YES];					// enable dial button
		[self namedDod];					// kill/restart named to dump cache
		}
				
    return self;
}
//*****************************************************************************
//
// 		run the preLink or preUnLink scripts, proceed if exit status true 
//
//*****************************************************************************

- runScript:(const char *)type  
{
id exitStatus = self;
int fd;

	strncpy(Path, [[NXBundle mainBundle] directory], MAXPATHLEN);
	strncat(Path, "/", MAXPATHLEN - strlen(Path));
	strncat(Path, type, MAXPATHLEN - strlen(Path));
	if(fd = open(Path, O_RDONLY) != -1)			//	if script
		{
		close(fd);
		[self openMainWin];						// open window
		if(system(Path) != 0)					// exec script, test exit
			exitStatus = nil;
		strcpy(Path, type);
		if(!exitStatus)
			strcat(Path," script exit status prevents requested action\n");
		else
			strcat(Path, " script executed sucessfully\n");
		[commandView appendStringUseFixedFont:Path];
		}

    return exitStatus;
}
//*****************************************************************************
//
// 		Returns whether we can open a Gate doc (only one doc allowed)
//
//*****************************************************************************

- (BOOL)appAcceptsAnotherFile:sender 
{
    return acceptAnother;
}
//****************************************************************************
//
//		termination is allowed only if the user has selected it, subprocess 
//		failure should not cause GateKeeper to terminate
//
//****************************************************************************

- terminate:sender
{
	userWantsTermination = YES;

    return self;
}
//****************************************************************************
//
//		termination subprocess 
//
//****************************************************************************

- appWillTerminate:sender
{
    [theSubprocess terminate:sender];
	[self removeTimedEntry];
	[self namedReset:"/usr/etc/named"];		// replace our named with system's
	
    return self;
}
- showInfo:sender
{
    if(!infoPanel)
		[NXApp loadNibSection:"InfoPanel.nib" owner:self withNames:NO];
    [infoPanel center];
    [infoPanel makeKeyAndOrderFront:self];
	
    return self;
}
//****************************************************************************
//
//		show preferences panel   
//
//****************************************************************************

- preferences:sender
{
	if(!prefPanel) 
		[NXApp loadNibSection:"Preferences.nib" owner:self withNames:NO];
	[prefPanel makeKeyAndOrderFront:self];
		
    return self;
}
//************************************************************************
//
//		show NXHelpPanel 
//
//************************************************************************

- showHelpPanel:sender 
{
    if(!helpPanel)
		helpPanel = [NXHelpPanel new];
    [helpPanel display];
    [helpPanel makeKeyAndOrderFront:self];
	
    return self;
}
//************************************************************************
//
//		show the timer 
//
//************************************************************************

- showTimerPanel:sender 
{
    return [theTimer showTimerPanel:self];
}
//*****************************************************************************
//
// 		called by timer in order to pass us appIcon time string 
//
//		also used in determining inactivity timeout (since this is called
//		once per minute during ppp sessions).
//
//*****************************************************************************

- showMenuTimer:(char *)buffer 
{
	[timeCell setStringValue:buffer];
	[theIconView setImage:onImage];
	[iTimer inactivityTimeout];

	return self; 
}
//*****************************************************************************
//
// 		set inactivity timer's timeout Ivar and write its value to ddb 
//
//*****************************************************************************

- setTimeout:(int)minTillTimeout 
{
	[iTimer setTimeout:minTillTimeout];

	return self; 
}
//************************************************************************
//
//		create new Gate Document with default files inside.
//
//************************************************************************

- newDoc:sender
{
	theSavePanel = [SavePanel new];
	[theSavePanel setTitle:"New"];
	[theSavePanel setRequiredFileType:filter[0]]; // append ext if not supplied
	if(NX_OKTAG == [theSavePanel runModalForDirectory:
						NXGetDefaultValue([NXApp appName], "path") file:NULL])
		{
    	mkdir([theSavePanel filename], 0777);		// create .Gate doc wrapper
		[self addDefaultFiles:[theSavePanel filename]];
		[self editGateDoc:[theSavePanel filename]];
		}

    return self;
}
//************************************************************************
//
//		execute pppd with info from .Gate document
//
//************************************************************************

- openDoc:sender
{
	theOpenPanel = [OpenPanel new];
	[theOpenPanel setTitle:"Link"];
	if(NX_OKTAG == [theOpenPanel runModalForDirectory:
						NXGetDefaultValue([NXApp appName], "path") file:NULL 	
							types:filter])
		{
		strcpy(Path, [theOpenPanel filename]);
		strcat(Path, olist[1]);
		if([self readable:Path])
			[self app:self openFile:[theOpenPanel filename] type:NULL];
		if(!NXWriteDefault([NXApp appName],"path",[theOpenPanel filename]))
			NXRunAlertPanel(0,"Error writing to defaults database",0 ,0, 0);	
		}

    return self;
}
//************************************************************************
//
//		edit an existing .Gate document
//
//************************************************************************

- editDoc:sender
{
	theOpenPanel = [OpenPanel new];
	[theOpenPanel setTitle:"Edit"];
	if(NX_OKTAG == [theOpenPanel runModalForDirectory:
						NXGetDefaultValue([NXApp appName], "path") file:NULL 	
							types:filter])
		[self editGateDoc:[theOpenPanel filename]];

    return self;
}
//************************************************************************
//
//		edit an existing .Gate document using built in editor
//
//************************************************************************

- editGateDoc:(const char *)aGateDoc
{
	if(!NXWriteDefault([NXApp appName],"path", aGateDoc))
		NXRunAlertPanel(0, "Error writing to defaults database", 0, 0, 0);	
	strcpy(Path, aGateDoc);
	strcat(Path, olist[0]);				// file name inside of dir
	if([self readable:Path])
		{
		if(!theOptionsEditor)					// open edit doc window
			[NXApp loadNibSection:"EditDoc.nib" owner:self withNames:NO];
		[theOptionsEditor parseOptionsFile:Path];
		[[theOptionsEditor editorWindow] setTitleAsFilename:Path];
		[[theOptionsEditor editorWindow] center];
		[[theOptionsEditor editorWindow] makeKeyAndOrderFront:self];
		}

    return self;
}
//************************************************************************
//
//		returns whether a Gate doc may be opened or edited by real user
//
//************************************************************************

- readable:(const char *)nameOfFile
{
FILE *fp;

	if ((fp = fopen(nameOfFile, "r+")) == NULL)
		NXRunAlertPanel(0,"Unable to open Gate doc",0, 0, 0);
	else
		{
		if(readable(fileno(fp)))
			{
			fclose(fp);
    		return self;				// we have permission
			}
		NXRunAlertPanel(0,"Access to Gate doc denied",0, 0, 0);
		fclose(fp);
		}

   	return nil;						// we do not have permission
}
//*****************************************************************************
//
// 		returns the Gate doc editor
//
//*****************************************************************************

- optionsEditor
{
    return theOptionsEditor;
}
//*****************************************************************************
//
// 			show the Tool Bar
//
//*****************************************************************************

- toolBar:sender
{
	if(!toolBar) 
		[NXApp loadNibSection:"ToolBar.nib" owner:self withNames:NO];
	[toolBar makeKeyAndOrderFront:self];

    return toolBar;
}
//************************************************************************
//
//		add our default options and pppup files to our new Gate document
//
//		paths for pppup file set from pref editor path
//
//************************************************************************

- addDefaultFiles:(const char *)nameOfFile
{
NXStream *Stream;
							// add options
	if(![[NXBundle mainBundle] getPath:Path forResource:"options" ofType:NULL])
		NXRunAlertPanel(0, "Error getting mainbundle path:aDef", 0, 0, 0);
	if([[NXApp delegate] readable:Path])
		{
		if((Stream = NXMapFile(Path, NX_READONLY)) == NULL)
			NXRunAlertPanel(0, "Unable to open options File", 0, 0, 0);
		else
			{
			strncpy(Path, nameOfFile, MAXPATHLEN - strlen(olist[0]));
			strncat(Path, olist[0], (strlen(olist[0]) + 1));
			if((NXSaveToFile(Stream, Path)) == -1)
				NXRunAlertPanel(0,"Error saving options to Gate doc" ,0,0,0);	
			NXClose(Stream);
			}
		}
	else
		NXRunAlertPanel(0,"Access to default options file denied", 0, 0, 0);

							// add pppup
	if(![[NXBundle mainBundle] getPath:Path forResource:"pppup" ofType:NULL])
		NXRunAlertPanel(0, "Error getting mainbundle path:aDef2", 0, 0, 0);
	if([[NXApp delegate] readable:Path])
		{
		if((Stream = NXMapFile(Path, NX_READONLY)) == NULL)
			NXRunAlertPanel(0, "Unable to open pppup default File", 0, 0, 0);
		else
			{
			strncpy(Path, nameOfFile, MAXPATHLEN - strlen(olist[1]));
			strncat(Path, olist[1], (strlen(olist[1]) + 1));
			if((NXSaveToFile(Stream, Path)) == -1)
				NXRunAlertPanel(0,"Error saving pppup to .Gate doc", 0, 0, 0);	
			NXClose(Stream);
			}
		}
	else
		NXRunAlertPanel(0,"Access to default pppup denied", 0, 0, 0);

						// add default Icon.tiff
	if(![[NXBundle mainBundle] getPath:Path 
										forResource:"Icon.tiff" ofType:NULL])
		NXRunAlertPanel(0, "Error getting Icon.tiff path", 0, 0, 0);
	if((Stream = NXMapFile(Path, NX_READONLY)) == NULL)
		NXRunAlertPanel(0, "Unable to open Icon.tiff File", 0, 0, 0);
	strncpy(Path, nameOfFile, MAXPATHLEN - strlen(olist[2]));
	strncat(Path, olist[2], (strlen(olist[2]) + 1));
	if((NXSaveToFile(Stream, Path)) == -1)
		NXRunAlertPanel(0,"Unable to save Icon.tiff to .Gate doc file",0,0, 0);	
	NXClose(Stream);

    return self;
}
//************************************************************************
//
// 		icon animation
//
//************************************************************************
 
- _animate
{
static int Index = 0;
static BOOL reverse = NO;

	if(standbyGate && Index < 0) 		// app running but not dialing
		{
		[self removeTimedEntry];
		Index = 9;
		}
	if(Index < 0)						// restarted before standby was reached
		{
		reverse = NO;
		Index++;
		}
	if(openGate)						// app stops dialing
		{
		Index = 4;
		reverse = YES;
		openGate = NO;
		}
	[theIconView setImage:[imageList objectAt:Index]]; 
	if(Index >= 8)
		{
		if(reverse)
			Index = -1;
		else
			Index = 5;
		}
	else
		{
		if(reverse)
			Index--;
		else
			Index++;
		}
			
	return self;
}
//************************************************************************
//
// 		return the app icons view
//
//************************************************************************
 
- appIconView
{
	return 	theIconView;					// enable link button
}
//************************************************************************
//
// 		called by DO server when remote manual Dial Tool's ports are 
//		invalidated
//
//************************************************************************
 
- DOFinished
{
	return 	[self UnLink:self];					// enable link button
}
//************************************************************************
//
// 		remove the timed entry when connection is made or we exit
//
//************************************************************************
 
- removeTimedEntry
{
	if (aniTimeTag)  
		DPSRemoveTimedEntry (aniTimeTag);
	aniTimeTag = 0;
		
	return self;
}
//*****************************************************************************
//
// 		opens the GateKeeper main window if not already open 
//
//*****************************************************************************

- openMainWin
{								// open GateKeeper main window
	if(strcmp(NXGetDefaultValue([NXApp appName], "DisplayDiagnostics"),"YES") 	
		== 0)
		{									// open main window 
    	if(!gateMWin)
			{
			[NXApp loadNibSection:"GateWindow.nib" owner:self withNames:NO];
			if(![gateMWin setFrameUsingName:[gateMWin title]])	// remem sz,loc
				[gateMWin setFrameAutosaveName:[gateMWin title]];
			}		
		[gateMWin makeKeyAndOrderFront:self];			// location, put in ddb
		}

	return self;
}
//************************************************************************
//
// 	free simply gets rid of everything we created
// 	This is how nice objects clean up.
//
//************************************************************************

- free
{
	[timeCell free];
	if(onImage)
		[onImage free];
	if(imageList)
		[imageList free];
	if(toolBar) 
		[toolBar free];
	if(iTimer) 
		[iTimer free];

    return [super free];
}
//*****************************************************************************
//
// 		Attempt to reset syslogd daemon.  Failing that, try starting it. 
//
//*****************************************************************************

- syslogdReset
{
FILE *ff; 
int pid;
					
    if((ff = fopen( "/etc/syslog.pid", "r")) != NULL)
		{
    	if( fscanf( ff, "%d", &pid) < 1)
        	perror("Unable to read pid from etc/syslog.pid\n");
		else
			{
			if( kill( pid, SIGHUP) != -1) 		// tell syslogd to reconfigure
				{
				fclose( ff);

				return self;					// report success
				}
			else			// if app was not dock autoLaunched
    			if(strcmp(NXGetDefaultValue(
								[NXApp appName],"NXAutoLaunch"),"NO") == 0) 
					perror("error reconfiguring syslogd");
			}
    	}
	else
		perror("Unable to open /etc/syslog.pid\n");
	
    return [self syslogdRun];		// failed to reconfig so restart syslogd
}
//*****************************************************************************
//
// 		Terminante and then run the syslogd daemon 
//
//*****************************************************************************

- syslogdRun
{
int pid;
FILE *ff; 
					
    if((ff = fopen( "/etc/syslog.pid", "r")) != NULL)
		{
    	if( fscanf( ff, "%d", &pid) < 1)
        	perror("Unable to read pid from etc/syslog.pid\n");
		else
			if( kill( pid, SIGTERM) == -1)
				{
    			if(strcmp(NXGetDefaultValue(	
								[NXApp appName],"NXAutoLaunch"),"NO") == 0) 
					perror("error killing syslogd");
				}
		}
	switch (pid = vfork())	// create new process 
		{							
		case -1:			// error
		
			perror("Error during vfork syslogdReset.");
			return self;
			
		case 0:  			// child -- Vfork returns 0 in the child's context

			execl("/usr/etc/syslogd", "syslogd", 0);
			perror("vfork error restarting syslogd"); 
			exit(1);

		default:			// parent --  vfork returns pid of the child  		
			
			break;	
		}		
	
    return self;
}
//*****************************************************************************
//
// 		prepare ourself to be triggered by named when dial on demand is
//		necessary 
//
//*****************************************************************************

- DialOnDemand					
{
port_t	server_port;
kern_return_t	r;

 							// if dial on demand is "ON"
	if(strcmp("YES", NXGetDefaultValue([NXApp appName], "DoD")) == 0)	
		{			
							// allocate a port for this task, ret in arg 2
		r = port_allocate(task_self(), &server_port);
		if (r != KERN_SUCCESS) 
			{
			mach_error("port_allocate failed", r);
			exit(1);
			}				// Register with the Network Name Server.
		r = netname_check_in(name_server_port, GK_DNS_SERVER, PORT_NULL, 
																server_port);
		if (r != KERN_SUCCESS) 
			{
			mach_error("netname_check_in failed", r);
			exit(1);
			}
						// create dial on demand server thread and detach it
			cthread_detach(cthread_fork((cthread_fn_t)server_loop, 
														(any_t)server_port));
		[self namedDod];	// replace named with dial on demand trigger named
		}
	
    return self;
}
//*****************************************************************************
//
// 		Attempt to kill and restart named daemon.  Resets RR's cache.
//
//*****************************************************************************

- namedReset:(const char *)buffer
{
FILE *ff; 
int pid;
 							// if dial on demand is "ON"
	if(strcmp("YES", NXGetDefaultValue([NXApp appName], "DoD")) == 0)
		{	
		if((ff = fopen( "/etc/named.pid", "r")) != NULL)
			{
			if( fscanf( ff, "%d", &pid) < 1)
				perror("Unable to read pid from etc/named.pid\n");
			else
				{
				if( kill( pid, SIGTERM) == -1) 			// kill named
					perror("error killing named");
				}
			}
		else
			perror("Unable to open /etc/named.pid\n");
		fclose( ff);
		system(buffer);				// start GateKeeper named 
		}
	
    return self;
}
//*****************************************************************************
//
// 		replace the system named with our own which will inform us when a
//		dial on demand is necessary 
//
//*****************************************************************************

- namedDod
{					
	if(!strcpy(Path, [[NXBundle mainBundle] directory]))	// app's home dir
		NXRunAlertPanel(0,"Error getting mainbundle path:nDod",0, 0, 0);
	else
		{
		strcat(Path,"/named");
		[self namedReset:Path];		// kill named and replace with our own
		}
	
    return self;
}
//*****************************************************************************
//
// 		search /etc/syslog.conf to find the location of the named pipe used 
//		in recieving the output from pppd/syslog 
//
//*****************************************************************************

- fifo
{
static char del1[] = {". =:|\t\r\n"}, del2[] = {" \t\r\n"};
char *fifoName = NULL;

	if(!Parser)									
		Parser = [[Parse alloc] init];			// create parser to find port
	[[Parser setKey1:"local2"] setKey2:"debug"];		// set search pattern  
	[[Parser setDelim1:del1] setDelim2:del2];			// and delimiters

	if(fifoName = [Parser parseFile:"/etc/syslog.conf"])
		{
		if(!NXWriteDefault([NXApp appName], "locFIFO", fifoName))
			NXRunAlertPanel(0,"Error setting defaults database",0,0,0);
		free(fifoName);
		}
	else
		NXRunAlertPanel(0, "Warning, unable to determine the FIFO's path from information stored in your /etc/syslog.conf file", 0, 0, 0);

    return self;
}
//*****************************************************************************
//
// 		search /etc/remote to find out what port the modem is at based on 
//		preferences value for "tip host".
//
//*****************************************************************************

- port
{
static char delimits[] = {"/ =:|\t\r\n"};
char *portName = NULL, *keyW;

	if(!Parser)									
		Parser = [[Parse alloc] init];			// create parser to find port
	keyW = NXCopyStringBuffer(NXGetDefaultValue([NXApp appName],"tipHost"));
	[[Parser setKey1:keyW] setKey2:"dev"];				// set search pattern 
	[[Parser setDelim1:delimits] setDelim2:delimits];	// and delimiters

	if(portName = [Parser parseFile:"/etc/remote"])
		{
		if(!NXWriteDefault([NXApp appName], "modemPort", portName))
			NXRunAlertPanel(0,"Error setting defaults database",0,0,0);
		free(portName);
		}
	else
		NXRunAlertPanel(0, "Warning, tip host specified in preferences was not found in your /etc/remote file", 0, 0, 0);
	free(keyW);

    return self;
}
//*****************************************************************************
//
// 		checks the mail queue for mail awaiting delivery 
//
//*****************************************************************************

- (BOOL)mailInQueue 
{
int	eCntr = 0;
struct direct *dirp;
DIR *dp;
BOOL mail = NO;

    if ((dp = opendir("/usr/spool/mqueue")) != NULL)
		{
		while ((dirp = readdir(dp)) && eCntr < 3) 	// read dir and cnt entries
			{
			if(*dirp->d_name != '.')				// don't count if sys file
				eCntr++;
			}
		if(eCntr > 2)
			mail = YES;
		closedir(dp);
		}
	else
		NXRunAlertPanel(0, "Error opening Mail queue directory", 0, 0, 0);

	return mail;
}
//*********************** Window delegate methods *****************************
//*****************************************************************************
//
// 		This method is called whenever the user minituriazes our window.
//
//*****************************************************************************

- windowWillMiniaturize:sender toMiniwindow:miniwindow 
{
    return [sender setMiniwindowIcon:"miniWinIcon"];
}

@end

//************************************************************************
//
//		This fucntion is registered by DPSaddtimedentry.
//		It is subsequently called every period t as registered  
//		in arg 0 of DPSaddtimedentry.
//
//************************************************************************

static void Animate(DPSTimedEntry time_tag, double now, id self)
{
	[self _animate];
}
//*****************************************************************************
//
// 		c thread loop which listens for RPC telling us to link 
//
//*****************************************************************************

any_t server_loop(any_t port)
{
struct message msg, reply;
kern_return_t ret;
        
    while (TRUE)
		{        
				/* Receive a request from a client. */
		msg.head.msg_local_port = (port_t)port;
		msg.head.msg_size = sizeof(struct message);
		ret = msg_receive(&msg.head, MSG_OPTION_NONE, 0);
		if (ret != RCV_SUCCESS)							/* ignore errors */
			continue; 
	
					/* Feed the request into the server. */
			(void)nsRPC_server((msg_header_t *)&msg, (msg_header_t *)&reply);
	
				/* Send a reply to the client. */
		reply.head.msg_local_port = (port_t)port;
		ret = msg_send(&reply.head, MSG_OPTION_NONE, 0);
		if (ret != SEND_SUCCESS) 						/* ignore errors */
			continue; 
		}
	return 0;
}
//*****************************************************************************
//
// This function is called by nsRPC_server, which was created by MiG.
// It is NOT directly called by any client process.
//
//*****************************************************************************

kern_return_t pppUp(port_t server)
{
	[[NXApp delegate] Link:nil];	

    return KERN_SUCCESS;
}
