//************************************************************************
//
//	m.m.  
//
//		by	Felipe A. Rodriguez		
//
//	This is a Multi-tasked, Multi-threaded tool which acts as a frontend 
//	for tip, a rearend for Terminal.app and which avails itself to 
//	GateKeeper through the distributed objects mechanism.
//
//
//	This code is supplied "as is" the author makes no warranty as to its 
//	suitability for any purpose.  This code is free and may be distributed 
//	in accordance with the terms of the:
//		
//			GNU GENERAL PUBLIC LICENSE
//			Version 2, June 1991
//			copyright (C) 1989, 1991 Free Software Foundation, Inc.
// 			675 Mass Ave, Cambridge, MA 02139, USA
//
//************************************************************************

#import "GateTool.h"
#import <appkit/nextstd.h> 
#import <dpsclient/dpsNeXT.h>
#import <stdio.h>
#import <sys/types.h>
#import <sys/ioctl.h>
#import <signal.h>
#import <termios.h>
#import <objc/hashtable.h>
#import <remote/NXProxy.h>
#import <machkit/NXPort.h>

#define	PTY_TEMPLATE "/dev/pty??"
#define	PTY_LENGTH 11
#define	BUFFERSIZE 512
#define	MAXSTRPAIRS 60		// max num of expect/send str pairs in modemInit 

	id GateKeeperProxy;					// proxy for GateKeeper server
	id DOserver;						// id of GateTool's DO server
	id GateConnection;					
    FILE *fpToChild;
	int tty;							// controlling tty
    int fromChild;
    int childPid;
    int masterPty;						// file descriptor for master/slave pty
    int slavePty;
    int bufferCount;
	volatile int loopcntr;				// counter for steps in script file
	volatile int ScriptRunning = 1;		// On/Off for script mode 
	volatile int alarmArmed = 1;		// alarm is armed or disarmed
	volatile int initMODEM = 1;			// are we sending a single AT command?
	char *tmpNames[MAXSTRPAIRS]; 		// Ptr's for script steps
	char *tmpTypes[MAXSTRPAIRS];
    char outputBuffer[BUFFERSIZE];
		// struc for storing Users original Local Terminal Shell params 
	struct sgttyb orgp = {B57600, B57600,(char)0x7f,(char)0x15, (CBREAK|ANYP)};
	mutex_t lock;	// assures completion of new thread before we continue

any_t gateThread(any_t arg);
void childDidExit(void);
void fdHandler (int theFd);
void Init(char *tipHost);
void getptys (int *master, int *slave);
void sendScriptToModem(char *buffer);
void openScriptFile(char *path);
void strToScript(const char *buffer);
void initThreaded(void);
void catchalarm(int a);				// timeout handler while sending script

		// not declared in sys/types header where it s/b
#ifndef pid_t
	#define pid_t int
#endif	/* pid_t */
extern pid_t getppid(void);			


int main(int argc, char *argv[]) 
{
static int Child, nread;
static char buff[BUFFERSIZE];

	initThreaded();					// setup DO in new thread
				// open modem init'ing file	
	openScriptFile([GateKeeperProxy modemInit]);
	Init([GateKeeperProxy tipHost]);
    switch (Child = vfork())		// create new process without fully copying
   	 	{							// the address space of the old process
    	case -1:	// error
			perror("Error starting 2nd UNIX vfork.");
			break;
			
    	case 0:  	// child -- Vfork returns 0 in the child's context
			for(;;)				// Child reads user shell and writes to Pty
				{
				if((nread = read(0, buff, BUFFERSIZE)) < 0)
					perror("Error reading STDIN.");
				else if(nread == 0)
					break;				//EOF on STDIN indicates we're done
				if(write(fromChild, buff, nread) != nread)
					perror("Error writing to Master Pty.");
				}
			kill(getppid(), SIGTERM);	//NOTIFY parent we have recieved EOF
			break;						// and are terminating
			
    	default:	// parent --  vfork returns pid of the child in the 			
					//	 parent's context
			signal(SIGALRM, catchalarm);  // alarm in case expect str not ret'd
			for(;;)				// Parent reads Pty and writes to user shell
				{
				if((nread = read(fromChild, buff, BUFFERSIZE)) <= 0)
					break;				//EOF or ERROR
				if(write(1, buff, nread) != nread)
					perror("Error writing to stdout.");
				if(ScriptRunning)
					{buff[nread] = '\0'; sendScriptToModem(buff); }
				}
			kill(Child, SIGTERM);
			break;					// and are terminating
    	}
	childDidExit();
    exit(0);
}

void childDidExit(void)
    // cleanup after a child process exits
{
    if (childPid)
		{
				// stty  --  reset parameters of the users Terminal Shell  
		(void)ioctl(tty, TIOCSETP, (char *)&orgp);
		close(fromChild);
		fclose(fpToChild);
		childPid=0;		// specify that child is dead
		}
		
    return;
}
//*****************************************************************************
//
// 		GateServer thread 
//
//		This thread avails GateToolServer, passes it the fd of the pty 
//		and then awaits a distributed objects connection to the server.
//
//*****************************************************************************

any_t gateThread(any_t arg)
{
	GateConnection = [NXConnection registerRoot: DOserver
        				withName:"GateToolServer"];	// DO name of server
	[DOserver setPty:fromChild];				// pass fd of master pty
	[NXPort worryAboutPortInvalidation];		// send invalidate when we end
	mutex_unlock(lock);							// remove mutex lock
	[GateConnection run];

	return arg;
}
//*****************************************************************************
//
//		attempt to setup the pty pair
//
//*****************************************************************************

void getptys (int *master, int *slave)
{
    char device[PTY_LENGTH];
    char *block, *num;
    char *blockLoc; // specifies the location of block for the device string
    char *numLoc; 	// specifies the pty name with the digit ptyxD
    char *msLoc;	// specifies the master (ptyxx) or slave (ttyxx)
    struct sgttyb setp = {B57600, B57600, (char)0x7f,(char)0x15, (CRMOD|ANYP)};
    struct tchars setc = {CINTR, CQUIT, CSTART, CSTOP, CEOF, CBRK};
    struct ltchars sltc = {CSUSP, CDSUSP, CRPRNT, CFLUSH, CWERASE, CLNEXT};
    int	lset = (LCRTBS|LCRTERA|LCRTKIL|LCTLECH|LPENDIN|LDECCTQ);
    int	setd = NTTYDISC;		/* new tty discipline */
    
    strcpy(device, PTY_TEMPLATE); 	// string constants are not writable
    blockLoc = &device[ strlen("/dev/pty") ];
    numLoc = &device[ strlen("/dev/pty?") ];
    msLoc = &device[ strlen("/dev/") ];
    for (block = "pqrs"; *block; block++)	
		{									
		*blockLoc = *block;
		for (num = "0123456789abcdef"; *num; num++)
			{
			*numLoc = *num;
			*master = open(device, O_RDWR);	//Open device assign fd to master
			if (*master >= 0)				//If master 0 or >, open succesful
				{
				*msLoc = 't';
				*slave = open(device, O_RDWR);
				if (*slave >= 0)
					{
						/* set parameters -- stty */
					(void) ioctl(*slave, TIOCSETP, (char *)&setp);
						/* set special characters */
					(void) ioctl(*slave, TIOCSETC, (char *)&setc);
						/* set line discipline */
					(void) ioctl(*slave, TIOCSETD, (char *)&setd);
						/* set local special chars */
					(void) ioctl(*slave, TIOCSLTC, (char *)&sltc);
						/* set entire local mode word */
					(void) ioctl(*slave, TIOCLSET, (char *)&lset);
					
					return;
					}
				}
			} /* hunting through a bank of ptys */
		} /* hunting through blocks of ptys in all the right places */
		
    *master = -1;		//If we get here we were unable to open 
    *slave = -1;		//master and/or slave pty's 
}
//*****************************************************************************
//
// 		initializes UNIX process to be driven from script file
//
//*****************************************************************************

void Init(char *tipHost)
{
struct sgttyb setp = {B9600, B9600, (char)0x7f, (char)0x15, (CBREAK|ANYP)};

    int numFds, fd;	// for temporary use
    int processGroup;
    int pidChild;			// needed because childPid does not exist
							// until Subprocess is instantiated
    								// /dev/tty kernel synm for controlling tty
	tty = open("/dev/tty", O_RDWR);			// open the controlling tty 
	getptys(&masterPty,&slavePty);			// find and open psuedo terms
	if (masterPty <= 0 || slavePty <= 0)
		{
		perror("Error grabbing ptys for subprocess.");
		return;
		}
					// gtty  --  get parameters of the users Terminal Shell  
	(void)ioctl(tty, TIOCGETP, (char *)&orgp);
					// stty  --  set parameters of the users Terminal Shell  
	(void)ioctl(tty, TIOCSETP, (char *)&setp);
    switch (pidChild = vfork())		// create new process without fully copying
   	 	{							// the address space of the old process
    	case -1:	// error
			perror("Error starting UNIX vfork of subprocess.");
			return;
			
    	case 0:  	// child -- Vfork returns 0 in the child's context
			dup2(slavePty, 0);		// dup slavePty to be stdin
			dup2(slavePty, 1);		// dup slavePty to be stdout
				dup2(slavePty, 2);	// dup slavePty to be stderr
			numFds = getdtablesize();
			for(fd=3; fd < numFds; fd++)		// Child should close all open
				close(fd);						// fd's beyond our basic 3
			processGroup = getpid();			// return current pid
				// think -- tty was opened by parent so set pgrp of tty
				//	to that of ourselves
			ioctl(0, TIOCSPGRP, (char *)&processGroup);		
			setpgrp (0, processGroup);  //make curr process process grp leader
			execl("/usr/bin/tip","tip", tipHost, 0);
//			execl("/usr/bin/tip","tip","b9600", 0);
				// output str to std error, should never get here tho
			perror("vfork (child)"); 
			exit(1);
	    default:		// parent --  vfork returns pid of the child in the 		
						//				parent's context
			childPid = pidChild;
			close(slavePty);
			fpToChild = fdopen(masterPty, "w");		//return ptr to masterPty
			fromChild = masterPty;
			setbuf(fpToChild, NULL);				// no buffering

			return;
		}
}
//*****************************************************************************
//
// 		process send/expect str pairs to and from a MODEM.
//		
//		three keywords are used SEND (send, w/echo), EXPECT (expected reply)
//		and PASS (send, w/o echo)
//
//*****************************************************************************

void sendScriptToModem(char *buffer)
{
static char view[32];
static char strbuf[256];
static int chrcnt = 0, cntr = 0, strpos = 0;
static char minBuffer[2] = {"b"};

	alarm(4);										// turn on alarm
	while(*buffer && (strpos < 255))
		{
		if((*buffer != '\n' && *buffer != '\r' && *buffer != '\177' && 
				*buffer != ' ' && *buffer != '\021' && *buffer != '\023' && 	
					*buffer != '\a') || (chrcnt != 0))
			{strbuf[strpos++] = *buffer++; strbuf[strpos] = '\0';}
		else {strpos = 0; buffer++;}
		}
	if(strcmp("EXPECT", tmpNames[cntr]) == 0)
		{
		if(strcmp(strbuf, tmpTypes[cntr]) == 0)		// test for expected str
			{
			cntr++;
			strpos = 0;
			}
		}
	if((strcmp("SEND", tmpNames[cntr]) == 0) || 
			(strcmp("PASS", tmpNames[cntr]) == 0))
		{
		if(chrcnt == 0) 
			sprintf(view,"%s\r",tmpTypes[cntr]);
//		if(chrcnt != 0)										Diagnostics Use
//			write(1, strbuf, sizeof(strbuf));
		if(strcmp(minBuffer, strbuf) == 0 || chrcnt == 0)
			{
			minBuffer[0] = view[chrcnt++];		//send one chr at a time
			if(write(fromChild, minBuffer, 1) != 1)		// Write to Master Pty
				perror("Error writing to Master Pty.");		// slave side of 				
			}											// is connected to tip
		strpos = 0;
		if(view[chrcnt] == '\0')
			{
			chrcnt = 0;					//Reset chrcnt when view str sent 					
			cntr++;						//Increment Script step cntr	
			}
		else
			{ 
			if(view[0] == '\r') sendScriptToModem("\r");	//Echo to call
			if(strcmp("PASS", tmpNames[cntr]) == 0)			//ourselves if we
				sendScriptToModem(minBuffer);				//are sending nl	
			}												//or a password
		}
	if(cntr >= (loopcntr - 1))				// Have we sent entire script yet?
		{
		ScriptRunning = 0;					// turn off script read/write
		alarmArmed = 0;
		cntr = 0;
		if(initMODEM)						// if we just init'd the MODEM
			{
			[GateKeeperProxy showDialPanel];
			initMODEM = 0;			// single command mode once modem is init'd
			}
		}		

    return;
}
//*****************************************************************************
//
// 		open modemInit script file 
//
//*****************************************************************************

void openScriptFile(char *path)
{
int loopstat = 1;
FILE *LogFileStream;
static char size1[16], size2[64], colon[16];

	if((LogFileStream = fopen(path, "r")) == NULL)
		perror("Error opening script file.");
	else
		{ 
		fseek(LogFileStream, 0, SEEK_SET);	// seek beginning
		while((loopstat != EOF) && (loopcntr < (MAXSTRPAIRS/2)))			
			{								
			loopstat = fscanf(LogFileStream, "%s %[^;] %s", 
												size1, size2, colon);
			tmpNames[loopcntr] = NXCopyStringBuffer(size1);
			if(strcmp("RETURN", size2) == 0)
				{size2[0] = '\r'; size2[1] = '\0'; }
			tmpTypes[loopcntr++] = NXCopyStringBuffer(size2);
			}
		if((loopstat != EOF) && (loopcntr >= (MAXSTRPAIRS/2)))
			perror("Too many init str's in modemInit file.");
		if(fclose(LogFileStream))
			perror("Error closing script file.");
		}

    return;
}
//*****************************************************************************
//
// 		convert a spc delimited str into a script to be sent with func
//		sendScriptToModem(char *buffer)  		i.e. "PASS ~. "
//
//*****************************************************************************

void strToScript(const char *buffer)
{
static char *str, delim[] = {" "}, *argString;

	loopcntr = 2;									// reset loopcntr global
	argString = NXCopyStringBuffer(buffer);
	str = strtok(argString, delim);					// Parse string and test 
	if(str != NULL)									
		{											
		tmpNames[0] = NXCopyStringBuffer(str);
		str = strtok(NULL, delim);				// next token 
		if(str != NULL)
			tmpTypes[0] = NXCopyStringBuffer(str);
		}
	ScriptRunning = 1;							// script mode is on
	sendScriptToModem("\r");

    return;
}
//*****************************************************************************
//
// 		instantantiate the distributed objects server in a new thread
//		notify gatekeeper we are up
//
//*****************************************************************************

void initThreaded(void)
{
	lock = mutex_alloc();

 	DOserver = [[GateTool alloc] init];			// create new DOserver
	mutex_lock(lock);		// place mutex lock to be released in new thread
							// create and detach a new thread at func
	cthread_detach(cthread_fork((cthread_fn_t)gateThread, (any_t)0));
	mutex_lock(lock);		// place mutex lock, s/fail if new thread not oper
	GateKeeperProxy = [NXConnection connectToName:"GateKeeperServer"];
				// protocol tells DO sys this is a oneway call so that
	if(GateKeeperProxy)		// we do not generate an uncaught exception 
		[GateKeeperProxy setProtocolForProxy:@protocol(DOserver)];
	[GateKeeperProxy activateGateKeeper];
	mutex_unlock(lock);							// remove mutex lock

    return;
}
//*****************************************************************************
//
// 		called if alarm expires while awaiting a char or str while sending 
//		the modemInitialize commands file to the modem. 
//
//*****************************************************************************

void catchalarm(int a)
{
	if(alarmArmed)
		{
		perror("Timeout error while sending modemInit script to the modem");
		exit(-1);
		}

    return;
}



