//************************************************************************
//
//	Preferences.m.  
//
//		Controller for the preferences panel.
//
//		by	Felipe A. Rodriguez		
//
//
//	This code is supplied "as is" the author makes no warranty as to its 
//	suitability for any purpose.  This code is free and may be distributed 
//	in accordance with the terms of the:
//		
//			GNU GENERAL PUBLIC LICENSE
//			Version 2, June 1991
//			copyright (C) 1989, 1991 Free Software Foundation, Inc.
// 			675 Mass Ave, Cambridge, MA 02139, USA
//
//************************************************************************

#import "Preferences.h"
#import "SwapView.h"
#import "OptionsEditor.h"
#import "ToolBar.h"

		// used in creating a FIFO
	typedef unsigned short mode_t;
int mkfifo (const char *path, mode_t mode);



@implementation Preferences

//************************************************************************
//
//		sent to us after nib objects are unarchived and init'd 
//
//************************************************************************
 
- awakeFromNib
{
int i = 0;

	[swapView initPopUp];
	[swapView inspectName:"Switches"];
						// remem loc/size
	[[swapView window] setFrameAutosaveName:[[swapView window] title]]; 

	do  {								// if set in ddb turn check switch on
		if(strcmp(NXGetDefaultValue([NXApp appName], sList[i]), "YES") == 0)		
			[[swMatrix findCellWithTag:i] setIntValue:1];
		}
	while(sList[++i]);
	for(i = 0; rsList[i] != NULL; i++) 					// redial switches
		{								// if set in ddb turn check switch on
		if(strcmp(NXGetDefaultValue([NXApp appName], rsList[i]), "YES") == 0)		
			[[redialSwMatrix findCellWithTag:i] setIntValue:1];
		}
	[preTimeoutSw setIntValue:1];
	for(i = 0; cList[i]; i++)				// display default trigger str's
		[defaultsForm setStringValue:NXGetDefaultValue([NXApp appName],
															cList[i]) at:i];	
	for(i = 0; rfList[i]; i++)				// display redial trigger str's
		[redialForm setStringValue:NXGetDefaultValue([NXApp appName],
															rfList[i]) at:i];	

	[self setPPPForm:radioButtons];				// set ppp form strings
	[self timeout:self];				// set timeout circ slide/text field
	[self timeoutThreshold:self];		// set timeout circ slide/text field
		
    return self;
}
//************************************************************************
//
//		show pppEditor 
//
//************************************************************************

- show_pppEditor:sender 
{
	[self setPPPForm:radioButtons];				// set ppp form strings

    return self;
}
//***************************** Editor subview ***************************
//************************************************************************
//
//		 Edit ppp file button was pressed, open radio sel'd file in edit
//
//************************************************************************
 
- pppFileEdit:sender 
{						// if readable
	if([[NXApp delegate] 
					readable:[self pathOf:(int)[radioButtons selectedTag]]])
						// call edit to open radioButtons selected file
		[[Application workspace] openFile:[self pathOf:
					(int)[radioButtons selectedTag]] withApplication:"Edit"];
	
    return self;
}
//************************************************************************
//
//		 Edit example ppp file button was pressed, in Editor subview.
//
//************************************************************************

- pppExampleEdit:sender 
{						// appWrapper is mainBundle, return path to resources
						// bundled within				 
    if(![[NXBundle mainBundle]getPath:Path forResource:"Examples" ofType:NULL])
		NXRunAlertPanel(0, "Error getting mainbundle path:ExEd", 0, 0, 0);
	strcat(Path, exlist[(int)[radioButtons selectedTag]]);				
					// call edit to open file
	[[Application workspace] openFile:Path withApplication:"Edit"];
	
    return self;
}
//************************************************************************
//
//		send formcell a path based on the tag of the selected radio button.
//
//************************************************************************

- setPPPForm:sender
{
	[pppForm setStringValue:[self pathOf:(int)[sender selectedTag]] at:0];	
	if((2 < (int)[sender selectedTag]))
		[pppForm setEnabled:NO];  	
	else 
		[pppForm setEnabled:YES];

    return self;
}
//************************************************************************
//
//		 return the path of the item selected in flist[]
//
//************************************************************************

- (char *)pathOf:(int)i 
{
	if(i == 8)	// modemInit files in mainbundle
		{
		if(![[NXBundle mainBundle] getPath:Path forResource:flist[i] 
																ofType:NULL])
			NXRunAlertPanel(0, "Error getting mainbundle path: pthOf",0, 0, 0);
		}
	else
		strcpy(Path, NXGetDefaultValue([NXApp appName], flist[i]));
	
    return Path;
}
//***************************** defaults subview *************************
//************************************************************************
//
//		user has changed the path of one of the ppp files.
//
//************************************************************************

- pppFilesPathChanged:sender
{									// write an entry to defaults database
	if(!NXWriteDefault([NXApp appName], flist[(int)[radioButtons selectedTag]], 
								[sender stringValueAt:[sender selectedIndex]]))
		NXRunAlertPanel(0, "Error setting defaults database", 0, 0, 0);

    return self;
}
//************************************************************************
//
//		user has changed a control string, store change in ddb
//
//************************************************************************
 
- defaultsForm:sender
{
int i = (int)[sender selectedIndex];

				// write an entry to defaults database
	if(!NXWriteDefault([NXApp appName], cList[i], [sender stringValueAt:i]))
		NXRunAlertPanel(0, "Error setting defaults database", 0, 0, 0);
	if(i == 6)		// tip host has been changed, update our port ddb variable
		[[NXApp delegate] port];
	if(i == 5)		// HotList path has changed
		[[[NXApp delegate] toolBar:self] initHotListPath];

    return self;
}
//************************************************************************
//
//		user has changed a redial trigger string, store change in ddb
//
//************************************************************************
 
- redialForm:sender
{
int i = (int)[sender selectedIndex];

				// write an entry to defaults database
	if(!NXWriteDefault([NXApp appName], rfList[i], [sender stringValueAt:i]))
		NXRunAlertPanel(0, "Error setting defaults database", 0, 0, 0);

    return self;
}
//************************************************************************
//
//		Target method of defaults db entries for our preferences
//		check switches based on the tag assigned in IB to each switch.
//
//************************************************************************

- switch:sender
{
int tag = [sender selectedTag];
										// toggles entries YES/NO
	if(strcmp(NXGetDefaultValue([NXApp appName], sList[tag]),"NO") == 0)
		{			// write an entry to defaults database
 		if(!NXWriteDefault([NXApp appName], sList[tag], "YES"))
			NXRunAlertPanel(0, "Error setting defaults database", 0, 0, 0);
		else 
			{			// Display Diagnostics was toggled, and if using fifo
			if((tag == 0 || tag == 7) && (strcmp(NXGetDefaultValue(
								[NXApp appName],"useFIFO"), "YES") == 0))				
				[self makeFIFO];
			if(tag == 6)				// Dial on Demand was enabled
				[[NXApp delegate] DialOnDemand];		// prepare for dial on 
			}
		}
	else 
		{
		if(tag == 6)			// Dial on Demand was disabled, revert to
			[[NXApp delegate] namedReset:"/usr/etc/named"];		// system named
		if((tag == 7) && (strcmp(NXGetDefaultValue(
						[NXApp appName], "DisplayDiagnostics"), "YES") == 0))				
			[self makeFile];	// if previously using FIFO create a log file
		if(!NXWriteDefault([NXApp appName], sList[tag], "NO"))
			NXRunAlertPanel(0, "Error setting defaults database", 0, 0, 0);	
		}

    return self;
}
//*****************************************************************************
//
// 		target of redial panel's switch matrix 
//
//*****************************************************************************

- redialSwitch:sender;
{
	if((BOOL)[sender intValue])	
		{
		if(!NXWriteDefault([NXApp appName],rsList[[sender selectedTag]],"YES"))
			NXRunAlertPanel(0, "Error writing to defaults database", 0, 0, 0);	
		}
	else
		{
		if(!NXWriteDefault([NXApp appName], rsList[[sender selectedTag]],"NO"))
			NXRunAlertPanel(0, "Error writing to defaults database", 0, 0, 0);	
		}

    return self;
}
//*****************************************************************************
//
// 		make the FIFO 
//
//*****************************************************************************

- makeFIFO
{								// find FIFO path per /etc/syslog.conf
	[[NXApp delegate] fifo];	
								// remove FIFO and then remake it
	if((strlen(NXGetDefaultValue([NXApp appName],"locFIFO")) + 20)< MAXPATHLEN)
		{
		strcpy(Path, "Create a FIFO at ");				
		strcat(Path, NXGetDefaultValue([NXApp appName],"locFIFO"));
		strcat(Path, " ?");
		if(NXRunAlertPanel(0, Path, "Yes", "No", 0) == NX_OKTAG)
			{
			unlink(NXGetDefaultValue([NXApp appName],"locFIFO"));		
			mkfifo(NXGetDefaultValue([NXApp appName], "locFIFO"),(mode_t) 644);
			[[NXApp delegate] syslogdRun];			// restart syslogd
			}
		}
	else
		NXRunAlertPanel(0, "Error FIFO path is too long", 0, 0, 0);

    return self;
}
//*****************************************************************************
//
// 		make the Log File
//
//*****************************************************************************

- makeFile
{
int fd;
								// find path per /etc/syslog.conf
	[[NXApp delegate] fifo];	
								// remove FIFO and then remake it
	if((strlen(NXGetDefaultValue([NXApp appName],"locFIFO")) + 27)< MAXPATHLEN)
		{
		strcpy(Path, "Create a ppp Log File at ");				
		strcat(Path, NXGetDefaultValue([NXApp appName],"locFIFO"));
		strcat(Path, " ?");
		if(NXRunAlertPanel(0, Path, "Yes", "No", 0) == NX_OKTAG)
			{
			unlink(NXGetDefaultValue([NXApp appName],"locFIFO"));
			if((fd = open(NXGetDefaultValue([NXApp appName],"locFIFO"), 
								O_RDWR|O_CREAT|O_TRUNC,(mode_t) 644)) != -1)
				close(fd);		
			[[NXApp delegate] syslogdRun];			// restart syslogd
			}
		}
	else
		NXRunAlertPanel(0, "log file path is too long", 0, 0, 0);

    return self;
}
//*****************************************************************************
//
// 		target of timeout circular slider 
//
//*****************************************************************************

- timeout:sender
{
int i;

	if(sender == self) 							// if initial display of pref
		i = atoi(NXGetDefaultValue([NXApp appName], "iTimeout"));
	else										// else user changed control
		{
		i = [sender intValue];
		[[NXApp delegate] setTimeout:i];		// set timeout in Coordinator
		}
	if(i == 0)
		[timeoutField setStringValue:"Off"];
	else
		[timeoutField setIntValue:i];
	[timeoutCircSlider setIntValue:i];

    return self;
}
//*****************************************************************************
//
// 		target of pre timeout switch 
//
//*****************************************************************************

- setPreTimeout:sender
{
	if((BOOL)[sender intValue])
		{
 		if(!NXWriteDefault([NXApp appName], "preTimeout", "YES"))
			NXRunAlertPanel(0, "Error setting defaults database", 0, 0, 0);
		}
	else 
		{
		if(!NXWriteDefault([NXApp appName], "preTimeout", "NO"))
			NXRunAlertPanel(0, "Error setting defaults database", 0, 0, 0);	
		}

    return self;
}
//*****************************************************************************
//
// 		target of timeout threshold circular slider 
//
//*****************************************************************************

- timeoutThreshold:sender
{
int i;

	if(sender == self)		// if initial display of pref
		i = atoi(NXGetDefaultValue([NXApp appName], "iTimeThreshold"));
	else										// else user changed control
		{
		i = [sender intValue];
		sprintf(Path,"%d",(int)i);
		if(!NXWriteDefault([NXApp appName], "iTimeThreshold", Path))
			NXRunAlertPanel(0, "Error setting defaults database", 0, 0, 0);	
		}											
	[timeoutThreshField setIntValue:i];
	[timeoutThreshCircSlider setIntValue:i];

    return self;
}
//*********************** text field matrix delegation ************************
//*****************************************************************************
//
// 		as text delegate we recieve this when textfield cell is being edited 
//
//*****************************************************************************

- (BOOL)textWillChange:textObject
{
	[[swapView window] setDocEdited:YES];	// indicate change is not saved

    return NO;
}
//*****************************************************************************
//
// 		text delegate receives when editing of textfield cell is ended 
//
//*****************************************************************************

- textDidEnd:textObject endChar:(unsigned short)whyEnd
{
    if ((whyEnd == 16) || (whyEnd == 17)) 
		[[swapView window] setDocEdited:NO];	// indicate change is saved

    return self;
}

@end

//*****************************************************************************
//
// 		create FIFO at designated path with mode set to 
//
//*****************************************************************************

int mkfifo (const char *path, mode_t mode)
{
	return mknod (path, mode | S_IFIFO, 0);
}
